@@ -101,7 +101,7 @@ def read_raster(data, band=1, window=None, window_crs=None, mask_geometry=False,
         if window is None:
             shape = f.shape
             if len(f.indexes) > 1:
-                data = np.ma.filled(f.read_band(band))
+                data = np.ma.filled(f.read(band))
             else:
                 data = np.ma.filled(f.read())
             affine = f.transform
@@ -121,7 +121,7 @@ def read_raster(data, band=1, window=None, window_crs=None, mask_geometry=False,
             # If window crs not specified, assume it is in raster crs
             ix_window = f.window(*window)
             if len(f.indexes) > 1:
-                data = np.ma.filled(f.read_band(band, window=ix_window))
+                data = np.ma.filled(f.read(band, window=ix_window))
             else:
                 data = np.ma.filled(f.read(window=ix_window))
             affine = f.window_transform(ix_window)
